#include "chpstick_constants.h"
#include <Servo.h>
#include <chpstick_4BeatWalking>
#include <chpstick_ServoMath>

// necessary servo objects
Servo HFR; // name convention:
Servo KFR; // char 1 2 3
Servo HFL; //      | | |
Servo KFL; //      | | -- R/L = right / left
Servo HRL; //      | ---- F/R = front / rear
Servo KRL; //      ------ H/K = hip   / knee  
Servo HRR; //
Servo KRR; //
Servo EYE; // yeah right... EYE servo does not follow the convention ;)

//  IR distance sensor
int EYEsensor = A0;  // analog pin for input
int threshold = 400; // raw measurement data threshold for obstacle avoidance     

void setup(){
  //hips
  HFR.attach(idHFR); // see constants.h for id reference
  HFL.attach(idHFL);
  HRL.attach(idHRL);
  HRR.attach(idHRR);
  
  //legs
  KFR.attach(idKFR);
  KFL.attach(idKFL);
  KRL.attach(idKRL);
  KRR.attach(idKRR);
  
  // head
  EYE.attach(idEYE); 
  pinMode(EYEsensor, INPUT);

  // start position
  moveServo(HFR, idHFR, 90, 1);
  moveServo(KFR, idKFR, FTG, 1);
  moveServo(HFL, idHFL, 90, 1);
  moveServo(KFL, idKFL, FTG, 1);
  moveServo(HRL, idHRL, -90, 1);
  moveServo(KRL, idKRL, FTG, 1);
  moveServo(HRR, idHRR, -90, 1);
  moveServo(KRR, idKRR, FTG, 1);
  moveServo(EYE, idEYE, EYE_center, 1); 
  
  Serial.begin(115200);
}

void loop(){
	
	if(obstacleFound(threshold)){
		// oh sh** something in the way
		obstacleScan();		// lets scan environment
		reverse4Beat(0, 1);  // crawl back 2 steps to escape
		reverse4Beat(0, 1);
		
		// where to go? random by now
		// obstacleScan should determine the escape direction in future
		switch (random(1,5)){ 
		case 1:
			for (int i = 0; i <4; i++){
				bodyTurnRight(0,1);
			}
			break;
		case 2:
			for (int i = 0; i <4; i++){
				bodyTurnLeft(0,1);
			}
			break;
		case 3:
			for (int i = 0; i <4; i++){
				crabLeft4Beat(0,1);
			}
			break;
		case 4:
			for (int i = 0; i <4; i++){
				crabRight4Beat(0,1);
			}
			break;
		}
	}else{
		// nothing in front of the sensor ... so keep beating forward
		forward4Beat(0, 1);
	}
}

boolean obstacleFound(int thold){
	int distance = 0; // reset distance for each call
	
	// average 5 measurements
	for (int i = 0; i < 5; i++){
		distance += analogRead(EYEsensor);
		delay(2);
	}
	distance = distance / 5;
	
	
	Serial.print("obstacle > ");
	Serial.print(threshold);
	Serial.print(" --> ");
	Serial.println(distance);
	
	// if distance gets greater than the threshold we are to close
	if (distance > thold){
		return true;
	}else{
		return false;
	}
}
void obstacleScan(){
	
	// TO DO
	// make use of this
	// right now just for show
	
	Serial.println("Scanning...");
	
	for (int i = EYE_min; i <=EYE_max; i++){
		moveServo(EYE, idEYE, i, 2);
		
		int distance = 0;
		
		for (int i = 0; i < 5; i++){
			distance += analogRead(EYEsensor);
			delay(2);
		}
		distance = distance / 5;
		
		Serial.print("...");
		Serial.print(distance);
		Serial.print(" @ ");
		Serial.print(i);
		Serial.println(" deg");
	}
	moveServo(EYE, idEYE, EYE_center, 1);
}
